function computeLPDImages(extremePoints, SSS, octaveOffset, basePath)
% computeLPDImages(extremePoints, SSS, octaveOffset, basePath)
%
% DESCRIPTION:
%  Generates images of the stereographic projection of the regions around
%  each extrema of the spherical difference of Gaussians. In addition, it
%  generates also two text files: planarData.txt and omniData.txt. These 
%  files contain the filename of each generated image and the coordinates 
%  and scale of the extreme in the planar projection and in the 
%  omnidirectional image respectively.
%  Using the data files and the generated images, Local Planar Descriptors 
%  (LPD) can be extracted. These LPD can be computed as a SIFT descriptor (a
%  histogram of orientations) calculated on the center of each image using 
%  the sigma's indicated in planarData.txt.
%  We do not provide directly Local Planar Descriptors because there is not 
%  a standard convention for the arrangement of the bins of the histogram of 
%  orientations of a SIFT descriptor. For this reason, we provide the images 
%  where the histogram of orientations can be computed, and this way you can 
%  use your own implementation in order to make Local Planar Descriptors 
%  compatible with your own extracted SIFT descriptors of a planar image.
%  If you do not have your own implementation of SIFT, you can use the 
%  implementation of Andrea Vedaldi that can be downloaded at
%  http://www.vlfeat.org/~vedaldi/code/sift.html
%  See Section 3.4 of the REFERENCE for further details.
%
% VERSION:
%  1.1
%
% INPUT:
%  extremePoints  =  list of extrema of the spherical difference of 
%                    Gaussians (output of "findLocalExtremes")
%  SSS            =  spherical scale-space representation of the image
%                    (output of "sphericalScaleSpace")
%  octaveOffset   =  stage where the pyramid of the spherical scale-space
%                    representation of the image starts (see 
%                    "sphericalScaleSpace")
%  basePath       =  path where the ouput images are stored
%
% REFERENCE:
%  @TECHREPORT{CruzBogdPaquBierThir09,
%     Author = {Javier Cruz-Mota and Iva Bogdanova and Beno{\^\i}t Paquier and Michel Bierlaire and Jean-Philippe Thiran},
%     Institution = {Transport and Mobility Laboratory, Ecole Polytechnique F\'ed\'erale de Lausanne (EPFL)},
%     Title = {Scale Invariant Feature Transform on the Sphere: Theory and Applications},
%     Year = {2009}
%  }

% AUTHOR:
%  Javier Cruz-Mota, email: javier.cruz@epfl.ch
%  Copyright (C) 2011 EPFL (Ecole Polytechnique Fédérale de Lausanne)
%  Transport and Mobility Laboratoy (TRANSP-OR)
% 
% LICENSE:
%  SIFTS is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%  Please refer to the file COPYING for more information.

    tic
    disp('[SIFTS] Processing stereographic projections for LPD computation...');
 
    planarFile = fopen([basePath '/planarData.txt'], 'w+');
    omniFile = fopen([basePath '/omniData.txt'], 'w+');
    numPoints = length(extremePoints.octave);
    sigmas = extremePoints.sigma(:);
    R = 6*sqrt(2)*sigmas;

    for i=1:numPoints
        img = SSS.octaves(extremePoints.octave(i)-octaveOffset+1).octaveContents(extremePoints.stage(i)+2).image;
        [imgR, imgC, n] = size(img);
        [phi,theta] = sphgrid(imgR,imgC);

        numSpCol = ceil(R(i)*imgC/(2*pi));
        if(numSpCol>imgC/4)
            numSpCol=imgC/4;
            R(i) = pi/2-eps;
        end
        vecCol = (-numSpCol-0.5:numSpCol+0.5);
        colCenter = pi*((imgC+1)/imgC);
        col = colCenter+vecCol*2.0*pi/imgC;

        numSpRow = ceil(R(i)*imgR/pi);
        if(numSpRow>imgR/2)
            numSpRow=imgR/2;
            R(i) = pi/2-eps;
        end
        vecRow = (-numSpRow-0.5:numSpRow+0.5);
        rowCenter = pi/2.0;
        row = rowCenter+vecRow*pi/imgR;

        phiOrig = extremePoints.octaveOptPoints(i,2)*2.0*pi/imgC;
        thetaOrig = (2*extremePoints.octaveOptPoints(i,1)+1)*pi/(2.0*imgR);

        % Rotation of rowCenter-thetaOrig wrt y
        [rowAngle, colAngle] = computeRotatedCoords(    row, col, pi/2, pi/2, ...
                                                        rowCenter-thetaOrig);
        % Rotation of phiOrig-colCenter wrt z
        [rowAngle, colAngle] = computeRotatedCoordsVector(    rowAngle(:), colAngle(:), 0, 0, ...
                                                              phiOrig-colCenter);

        [cols,rows] = meshgrid( min(floor(colAngle*imgC/(2*pi))):max(ceil(colAngle*imgC/(2*pi))), ...
                                min(floor(rowAngle*imgR/pi)):max(ceil(rowAngle*imgR/pi)));

        indicatorMatrix = cols<=round(extremePoints.octaveOptPoints(i,2));
        for numLine=1:size(indicatorMatrix,1)
            [val,firstInd]=max(indicatorMatrix(numLine,:));
            indicatorMatrix(numLine,1:firstInd)=val;
            [val,firstInd]=min(indicatorMatrix(numLine,:));
            indicatorMatrix(numLine,firstInd:end)=val;
        end
        
        indPhis = find(indicatorMatrix);

        rows(rows==0) = 1;
        rows(rows==imgR+1) = imgR;

        cols(cols<1) = cols(cols<1)+imgC;
        cols(cols>imgC) = cols(cols>imgC)-imgC;
       
        numRows = length(rows(:,1));
        numCols = length(cols(1,:));

        S1 = img(rows+(cols-1)*imgR);
        
        % Computation of the new spherical coordinates with respect to the
        % detected point
        % NewTheta = dist(detected point, point)
        % cos (dist(detected point+NewTheta, point)) = cos^2 (newTheta)
        %        + sin^2 (newTheta) * cos(NewPhi) ==>
        % ==> NewPhi=acos((cos(dist(...))-cos^2(NewTheta))/sin^2(newTheta))
        
        thetas = sphericalDistance( phiOrig, phi(1,cols(:)), ...
                                    thetaOrig, theta(rows(:),1)');
                                
        thetas = reshape(thetas,numRows,numCols);

        % Avoiding problems due to simplification of the region that has to
        % be taken into account (a squared region is considered for "cols"
        % and "rows"). Some of the points at higher scales are discarded!
        if (~isempty(thetas(thetas>pi-pi/100)))
            continue;
        end
        
        sinThetas = sin(thetas);
        sinThetas2 = sinThetas.^2;
        cosThetas2 = cos(thetas).^2;
        a = zeros(numRows,numCols);
               
        for j=1:length(rows(:))
                a(j) = sphericalDistance(phiOrig, ...
                                      phi(rows(j),cols(j)), ...
                                      thetaOrig+thetas(j), ...
                                      theta(rows(j),cols(j)));
        end
        phis = acos(max(min((cos(a)-cosThetas2)./sinThetas2,1.0),-1.0));
        phis(indPhis) = -phis(indPhis);
        
        cartesianLimit = 2*tan((R(i)/sqrt(2))/2);
        cartesianSpace = 2*tan(pi/(2.0*imgR));

        [Xd,Yd] = meshgrid(-cartesianLimit:cartesianSpace:cartesianLimit,-cartesianLimit:cartesianSpace:cartesianLimit);
        xij = 2*tan(thetas/2).*sin(phis);
        yij = 2*tan(thetas/2).*cos(phis);
        
        image = griddata(xij,yij,255.0*S1,Xd,Yd,'linear');
        image(isnan(image)) = 0;
        image = uint8(image);
        
        fileName = sprintf('%s/projectedImage%05d.bmp',basePath,i);
        imwrite(image,fileName,'bmp');

        fprintf(planarFile,'%s %f %f %f \n',fileName,(length(Xd(1,:))-1)/2,(length(Yd(:,1))-1)/2,2*tan(sigmas(i)/2)/cartesianSpace);
        fprintf(omniFile,'%s %f %f %f \n',fileName,extremePoints.imgOptPoints(i,1),extremePoints.imgOptPoints(i,2),extremePoints.sigma(i));
    end
    fclose(planarFile);
    fclose(omniFile);
    t=toc;
    disp('[SIFTS] Done!');
    disp(sprintf('[SIFTS] Computation time: %f seconds\n',t));
    
end
