function [LSD, extremePointsOut] = computeLSD(extremePoints, SSS, numBinsPrincipalAngle, numBins, numSpatialDivisions)
% [LSD, extremePointsOut] = computeLSD(extremePoints, SSS, numBinsPrincipalAngle, numBins, numSpatialDivisions)
%
% DESCRIPTION:
%  Computes Local Spherical Descriptors of each considered extreme of the
%  spherical difference of Gaussians.
%  See Section 3.3 of the REFERENCE for further details.
%
% VERSION:
%  1.1
%
% INPUT:
%  extremePoints         =  list of extrema of the spherical difference of 
%                           Gaussians (output of "findLocalExtremes")
%  SSS                   =  spherical scale-space representation of the 
%                           image (output of "sphericalScaleSpace")
%  numBinsPrincipalAngle =  number of bins of the histogram of orientations
%                           for the computation of the principal
%                           orientation
%  numBins               =  number of bins of the descriptor in the 
%                           orientation dimension
%  numSpatialDivisions   =  number of divisions of each of the two spatial 
%                           dimensions for the computation of the
%                           descriptor
%
% OUTPUT:
%  LSD               =  list of Local Spherical Descriptors
%  extremePointsOut  =  "extremePoints" (input) with their corresponding
%                       principal orientation
%
% REFERENCE:
%  @TECHREPORT{CruzBogdPaquBierThir09,
%     Author = {Javier Cruz-Mota and Iva Bogdanova and Beno{\^\i}t Paquier and Michel Bierlaire and Jean-Philippe Thiran},
%     Institution = {Transport and Mobility Laboratory, Ecole Polytechnique F\'ed\'erale de Lausanne (EPFL)},
%	  Title = {Scale Invariant Feature Transform on the Sphere: Theory and Applications},
%     Year = {2009}
%  }


% AUTHOR:
%  Javier Cruz-Mota, email: javier.cruz@epfl.ch
%  Copyright (C) 2011 EPFL (Ecole Polytechnique Fédérale de Lausanne)
%  Transport and Mobility Laboratoy (TRANSP-OR)
% 
% LICENSE:
%  SIFTS is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%  Please refer to the file COPYING for more information.


    tic
    LSD = [];
    extremePointsOut = struct('octave',[],'stage',[],'sigma',[],'octavePoints',[],...
                              'octaveOptPoints',[],'imgPoints',[],'imgOptPoints',[],'gradientAngle',[]);
    extremesPerStagePerOctave = repmat(struct(extremePointsOut),[SSS.numOctaves, SSS.numStagesPerOctave-3]);

    disp('[SIFTS] Computing Local Spherical Descriptors...');

    for i = 1:SSS.numOctaves
        for j=1:SSS.numStagesPerOctave-3
            [ind]=find((extremePoints.octave==i+SSS.octavesOffset-1) & (extremePoints.stage==j-1));
            if(~isempty(ind))
                extremesPerStagePerOctave(i, j).octave = extremePoints.octave(ind);
                extremesPerStagePerOctave(i, j).stage = extremePoints.stage(ind);
                extremesPerStagePerOctave(i, j).sigma = extremePoints.sigma(ind);
                extremesPerStagePerOctave(i, j).octavePoints = extremePoints.octavePoints(ind,:);
                extremesPerStagePerOctave(i, j).octaveOptPoints = extremePoints.octaveOptPoints(ind,:);
                extremesPerStagePerOctave(i, j).imgPoints = extremePoints.imgPoints(ind,:);
                extremesPerStagePerOctave(i, j).imgOptPoints = extremePoints.imgOptPoints(ind,:);
                extremesPerStagePerOctave(i, j).gradientAngle = extremePoints.gradientAngle(ind);

                [LSDescriptor,extremesPerStagePerOctave(i, j)] = getLSD(extremesPerStagePerOctave(i,j), ...
                                                                            SSS.octaves(i).octaveContents(j+1).image, ...
                                                                            extremesPerStagePerOctave(i,j).sigma(1), ...
                                                                            numBinsPrincipalAngle, numBins, numSpatialDivisions);

                LSD = [LSD; LSDescriptor'];

                extremePointsOut.octave = [extremePointsOut.octave; extremesPerStagePerOctave(i, j).octave];
                extremePointsOut.stage  = [extremePointsOut.stage; extremesPerStagePerOctave(i, j).stage];
                extremePointsOut.sigma  = [extremePointsOut.sigma; extremesPerStagePerOctave(i, j).sigma];
                extremePointsOut.octavePoints = [extremePointsOut.octavePoints; extremesPerStagePerOctave(i, j).octavePoints];
                extremePointsOut.octaveOptPoints = [extremePointsOut.octaveOptPoints; extremesPerStagePerOctave(i, j).octaveOptPoints];
                extremePointsOut.imgPoints = [extremePointsOut.imgPoints; extremesPerStagePerOctave(i, j).imgPoints];
                extremePointsOut.imgOptPoints = [extremePointsOut.imgOptPoints; extremesPerStagePerOctave(i, j).imgOptPoints];
                extremePointsOut.gradientAngle = [extremePointsOut.gradientAngle; extremesPerStagePerOctave(i, j).gradientAngle];

                disp(sprintf('[SIFTS]\t Octave %d Stage %d DONE!',i+SSS.octavesOffset-1,j-1));
            end
        end
    end
    t=toc;
    numDescriptors = length(LSD);
    disp(sprintf('[SIFTS]\t %d Local Spherical Descriptors computed!', numDescriptors));
    disp('[SIFTS] Done!');
    disp(sprintf('[SIFTS] Computation time: %f seconds\n',t));
end
