function [ScaleSpace] = sphericalScaleSpace(Image,S,numOctaves,octaveOffset, sigma0, sigmaN)
% [ScaleSpace] = sphericalScaleSpace(Image,S,numOctaves,octaveOffset, sigma0, sigmaN)
%
% DESCRIPTION:
%  Computes the spherical scale-space representation of an omnidirectional
%  image mapped on the sphere
%
% VERSION:
%  1.1
%
% INPUT:
%  Image         =  omnidirectional input image mapped on the sphere
%  S             =  number of stages per octave
%  numOctaves    =  number of octaves
%  octaveOffset  =  offset of the first octave in the spherical scale-space
%                   representation of the image (0=the computation starts 
%                   with the original image; -1=the input image size is
%                   doubled before starting the computation)
%  sigma0        =  sigma of the first stage of octave 0
%  sigmaN        =  nominal sigma of the input image
%
% OUTPUT:
%  ScaleSpace  =  spherical scale-space representation of the
%                 omnidirectional input image

% AUTHOR:
%  Javier Cruz-Mota, email: javier.cruz@epfl.ch
%  Copyright (C) 2011 EPFL (Ecole Polytechnique Fédérale de Lausanne)
%  Transport and Mobility Laboratoy (TRANSP-OR)
% 
% LICENSE:
%  SIFTS is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%  Please refer to the file COPYING for more information.
%
% REFERENCE:
%  @TECHREPORT{CruzBogdPaquBierThir09,
%     Author = {Javier Cruz-Mota and Iva Bogdanova and Beno{\^\i}t Paquier and Michel Bierlaire and Jean-Philippe Thiran},
%     Institution = {Transport and Mobility Laboratory, Ecole Polytechnique F\'ed\'erale de Lausanne (EPFL)},
%     Title = {Scale Invariant Feature Transform on the Sphere: Theory and Applications},
%     Year = {2009}
%  }

    [Ny,Nx,n]=size(Image);
    
    tic
    octaveLevels = struct('stage',{},'sigma',{},'image',{});
    octave = struct('k',{},'octave',{},'octaveContents',{});
    ScaleSpace = struct('octavesOffset', {}, 'numOctaves',{}, 'numStagesPerOctave', {}, 'octaves', {});

    disp('[SIFTS] Computing spherical scale-space...');

    if (octaveOffset~=-1 && octaveOffset~=0)
        octaveOffset = 0;
        disp('[SIFTS]\t Invalid value for \"octaveOffset\", using 0 instead!');
    end
    
    k = 2^(1/S);

    sigma0S2 = sigma0*pi/Ny;
    
    % If the image has to be doubled in size, do it!
    if (octaveOffset==-1)
        L = imresize(Image, 2);
        L = L-min(min(L));
        L = L/max(max(L));
        msg = sprintf('[SIFTS]\t Computing octave %d:', octaveOffset);
        disp(msg);
        sigmaNS2 = (2*sigmaN)*pi/Ny;
        iterK = sqrt((sigma0S2/k)^2 - sigmaNS2^2);
        L = sphericalGaussianFilter(L,iterK);

        octaveLevels(1).stage = -1;
        octaveLevels(1).sigma = 0.5*sigma0S2/k;
        octaveLevels(1).image = L;
        iterK = (sigma0S2/k)*sqrt(k^2 - 1.0);

        for s=2:S+3
            L = sphericalGaussianFilter(L,iterK);
            octaveLevels(s).stage = s-2;

            octaveLevels(s).sigma = sqrt(octaveLevels(s-1).sigma^2+iterK^2);
            octaveLevels(s).image = L;
            if(s==S+1)
                tmpPreventAlias = L;
            end

            iterK = k*iterK;
        end

        octave(1).k = k;
        octave(1).octave = octaveOffset;
        octave(1).octaveContents = octaveLevels;
        
        disp(sprintf('[SIFTS]\t Octave %d done!',octaveOffset));
    end

    L = Image;
    skipAlias = 0;
    
    for j=1:numOctaves
        msg = sprintf('[SIFTS]\t Computing octave %d:', j-1);
        disp(msg);
        
        if(j==1)
            sigmaNS2 = sigmaN*pi/Ny;
            iterK = sqrt((sigma0S2/k)^2 - sigmaNS2^2);
            L = sphericalGaussianFilter(L,iterK);
        end

        octaveLevels(1).stage = -1;
        octaveLevels(1).sigma = (2^(j-1))*sigma0S2/k;
        iterK = octaveLevels(1).sigma*sqrt(k^2 - 1.0);
        if (~skipAlias)
            octaveLevels(1).image = L;
        else
            tmpAlias = L;
            for sa = 1:skipAlias
                tmpAlias = tmpAlias(2:2:end, 2:2:end);
            end
            octaveLevels(1).image = tmpAlias;
        end

        for s=2:S+3
            L = sphericalGaussianFilter(L,iterK);
            octaveLevels(s).stage = s-2;

            if (~skipAlias)
                octaveLevels(s).sigma = sqrt(octaveLevels(s-1).sigma^2+(iterK)^2);
                octaveLevels(s).image = L;
            else
                tmpAlias = L;
                for sa = 1:skipAlias
                    tmpAlias = tmpAlias(2:2:end, 2:2:end);
                end
                octaveLevels(s).image = tmpAlias;
                octaveLevels(s).sigma = sqrt(octaveLevels(s-1).sigma^2+(iterK)^2);
            end
            if(s==S+1)
                tmpPreventAlias = L;
            end
            iterK = k*iterK;
        end

        octave(j-octaveOffset).k = k;
        octave(j-octaveOffset).octave = j-1;
        octave(j-octaveOffset).octaveContents = octaveLevels;
        
        if( exp(-((Ny/(2^j))*(Ny/(2^j)+1)/8)*(sigma0S2/k)^2) <=exp(-1) )
            L = octaveLevels(S+1).image(2:2:end, 2:2:end);
        else
            skipAlias = skipAlias+1;
            L = tmpPreventAlias;
        end
 
        disp(sprintf('[SIFTS]\t Octave %d done!',j-1));
    end
    
    ScaleSpace(1).octavesOffset = octaveOffset;
    ScaleSpace(1).numOctaves = numOctaves-octaveOffset;
    ScaleSpace(1).numStagesPerOctave = S+3;
    ScaleSpace(1).octaves = octave;
    disp('[SIFTS] Done!');
    t=toc;
    disp(sprintf('[SIFTS] Computation time: %f seconds\n',t));

end
