function [E, T_noise] = errorTransform(Theta, X, sigma, P_inlier)
% [E, T_noise] = errorTransform(Theta, X, sigma, P_inlier)
%
% DESCRIPTION:
%  Computes the squared symmetric reprojection error. Inputs and outputs
%  are managed by the RANSAC toolbox (see its documentation for further
%  details).
%
% VERSION:
%  1.1
%

% AUTHOR:
%  Javier Cruz-Mota, email: javier.cruz@epfl.ch
%  Copyright (C) 2011 EPFL (Ecole Polytechnique Fédérale de Lausanne)
%  Transport and Mobility Laboratoy (TRANSP-OR)
% 
% LICENSE:
%  SIFTS is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%  Please refer to the file COPYING for more information.
%
% REFERENCE:
%  @TECHREPORT{CruzBogdPaquBierThir09,
%     Author = {Javier Cruz-Mota and Iva Bogdanova and Beno{\^\i}t Paquier and Michel Bierlaire and Jean-Philippe Thiran},
%     Institution = {Transport and Mobility Laboratory, Ecole Polytechnique F\'ed\'erale de Lausanne (EPFL)},
%     Title = {Scale Invariant Feature Transform on the Sphere: Theory and Applications},
%     Year = {2009}
%  }

    E = [];
    if (~isempty(Theta) && ~isempty(X))

        H = [Theta(1:3)';Theta(4:6)';Theta(7:9)'];

        X12 = H*X(1:3,:);
        for i=1:length(X(1,:))
            X12(:,i) = X12(:,i)/norm(X12(:,i));
        end
        
        X21 = H\X(4:6,:);
        for i=1:length(X(1,:))
            X21(:,i) = X21(:,i)/X21(3,i);
        end
        
        E1 = sum((X(1:3, :)-X21).^2, 1);
        E2 = sum((X(4:6, :)-X12).^2, 1);

        E = E1 + E2;
    end

    % compute the error threshold
    if (nargout > 1)

        if (P_inlier == 0)
            T_noise = sigma;
        else
            % Assumes the errors are normally distributed.            
            T_noise = sigma^2 * chi2inv_LUT(P_inlier, 6);
        end
    end
end
