function [phi,theta,phiValues,thetaValues] = computeRegions(extrema,imgR,imgC,octaveImage,R)
% [phi,theta,phiValues,thetaValues] = computeRegions(extrema,imgR,imgC,octaveImage,R)
%
% DESCRIPTION:
%  Computes theta and phi values of the points on the border of the 
%  interest region of each interest point (extrema of the difference of
%  Gaussians).
%
% VERSION:
%  1.1
%
% INPUT:
%  extrema      =  list of extrema of the spherical difference of Gaussians
%  imgR         =  number of rows of the spherical image
%  imgC         =  number of columns of the spherical image
%  octaveImage  =  1 if the regions have to be computed for a specific
%                  octave
%  R            =  array containing the radius of the regions of interest
%                  for each extreme
%
% OUTPUT:
%  phi          =  array of phi values of the spherical grid
%  theta        =  array of theta values of the spherical grid
%  phiValues    =  matrix containing phi values of the points of the 
%                  spherical grid on the border of the region of interest 
%                  of each interest point. Each row of the matrix 
%                  corresponds to an interest point.
%  thetaValues  =  matrix containing theta values of the points of the 
%                  spherical grid on the border of the region of interest 
%                  of each interest point. Each row of the matrix 
%                  corresponds to an interest point.

% AUTHOR:
%  Javier Cruz-Mota, email: javier.cruz@epfl.ch
%  Copyright (C) 2011 EPFL (Ecole Polytechnique Fédérale de Lausanne)
%  Transport and Mobility Laboratoy (TRANSP-OR)
% 
% LICENSE:
%  SIFTS is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%  Please refer to the file COPYING for more information.
%
% REFERENCE:
%  @TECHREPORT{CruzBogdPaquBierThir09,
%     Author = {Javier Cruz-Mota and Iva Bogdanova and Beno{\^\i}t Paquier and Michel Bierlaire and Jean-Philippe Thiran},
%     Institution = {Transport and Mobility Laboratory, Ecole Polytechnique F\'ed\'erale de Lausanne (EPFL)},
%     Title = {Scale Invariant Feature Transform on the Sphere: Theory and Applications},
%     Year = {2009}
%  }

    phi = 0:2*pi/imgC:2*pi*(1-1/imgC);
    theta = pi/(2*imgR):pi/imgR:pi*(1-1/(2*imgR));

    [numPoints] = size(R,1);
    
    t = 0:2*pi/50:2*pi;
    alpha = asin(R);

    x = sin(alpha)*cos(t);
    y = sin(alpha)*sin(t);
    z = cos(alpha)*ones(size(t));
    
    if(octaveImage)
        thetaP = theta(extrema.octavePoints(:,1))';
        phiP = phi(extrema.octavePoints(:,2))';
    else
        thetaP = theta(extrema.imgPoints(:,1))';
        phiP = phi(extrema.imgPoints(:,2))';
    end
    
    for i=1:numPoints
        xp=cos(thetaP(i))*x(i,:)+sin(thetaP(i))*z(i,:);

        z(i,:)=-sin(thetaP(i))*x(i,:)+cos(thetaP(i))*z(i,:);
        x(i,:)=cos(phiP(i))*xp-sin(phiP(i))*y(i,:);
        y(i,:)=sin(phiP(i))*xp+cos(phiP(i))*y(i,:);
    end
    
    phiValues = atan2(y,x);
    thetaValues = acos(z);
    
    for i=1:50
        for j=1:numPoints
            if(abs(phiValues(j,i+1)-phiValues(j,i))>pi)
                phiValues(j,i+1)=phiValues(j,i+1)+sign(phiValues(j,i))*2*pi;
            end
        end
    end

end
