function [newTheta, newPhi] = computeRotatedCoords(theta, phi, thetaCenter, phiCenter, angle)
% [newTheta, newPhi] = computeRotatedCoords(theta, phi, thetaCenter, phiCenter, angle)
%
% DESCRIPTION:
%  Computes the counterclockwise rotation of points in the spherical grid 
%  around a given axis
%
% VERSION:
%  1.1
%
% INPUT:
%  theta        =  theta values of the points that will be rotated
%  phi          =  phi values of the points that will be rotated
%  thetaCenter  =  theta coordinate of the axis of rotation
%  phiCenter    =  phi coordinate of the axis of rotation
%  angle        =  angle of rotation
%
% OUTPUT:
%  newTheta  =  theta values of the points after the rotation
%  newPhi    =  phi values of the points after the rotation

% AUTHOR:
%  Javier Cruz-Mota, email: javier.cruz@epfl.ch
%  Copyright (C) 2011 EPFL (Ecole Polytechnique Fédérale de Lausanne)
%  Transport and Mobility Laboratoy (TRANSP-OR)
% 
% LICENSE:
%  SIFTS is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%  Please refer to the file COPYING for more information.
%
% REFERENCE:
%  @TECHREPORT{CruzBogdPaquBierThir09,
%     Author = {Javier Cruz-Mota and Iva Bogdanova and Beno{\^\i}t Paquier and Michel Bierlaire and Jean-Philippe Thiran},
%     Institution = {Transport and Mobility Laboratory, Ecole Polytechnique F\'ed\'erale de Lausanne (EPFL)},
%     Title = {Scale Invariant Feature Transform on the Sphere: Theory and Applications},
%     Year = {2009}
%  }
    
    numR = length(theta);
    numC = length(phi);
    dim  = numR*numC;

    v = [reshape(sin(theta)'*cos(phi),[],1), reshape(sin(theta)'*sin(phi),[],1), reshape(repmat(cos(theta)',1,numC),[],1)];

    u = [sin(thetaCenter)*cos(phiCenter)*ones(dim,1), sin(thetaCenter)*sin(phiCenter)*ones(dim,1), cos(thetaCenter)*ones(dim,1)];

    cosAngle = cos(angle);
    sinAngle = sin(angle);
    cosAngleU = (1-cosAngle)*u(1,:);
    vRot = v*cosAngle+cross(u,v,2)*sinAngle+dot(u,v,2)*cosAngleU;

    newTheta = reshape(atan2(sqrt(vRot(:,1).^2 + vRot(:,2).^2),vRot(:,3)),numR,numC);
    newPhi = reshape(mod(atan2(vRot(:,2),vRot(:,1)),2*pi),numR,numC);
end
