function [descriptor] = computeRotatedDescriptor(regionAngs, regionMods, numSpatialDivisions, numOrientationDivisions)
% [descriptor] = computeRotatedDescriptor(regionAngs, regionMods, numSpatialDivisions, numOrientationDivisions)
%
% DESCRIPTION:
%   Computes the histogram of orientations corresponding to the Local
%   Spherical Descriptor
%
% VERSION:
%  1.1
%
% INPUT:
%  regionAngs               =  phase, with respect to the principal 
%                              orientation, of the points of the spherical 
%                              grid inside the region of interest for the 
%                              descriptor computation 
%  regionMods               =  distance to the interest point of the points 
%                              of the spherical grid inside the region of 
%                              interest for the descriptor computation
%  numSpatialDivisions      =  number of divisions in the spatial dimension
%  numOrientationDivisions  =  number of divisions in the orientation
%                              dimension
%
% OUTPUT:
%  descriptor  =  histogram of orientations corresponding to the Local
%                 Spherical Descriptor

% AUTHOR:
%  Javier Cruz-Mota, email: javier.cruz@epfl.ch
%  Copyright (C) 2011 EPFL (Ecole Polytechnique Fédérale de Lausanne)
%  Transport and Mobility Laboratoy (TRANSP-OR)
% 
% LICENSE:
%  SIFTS is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%  Please refer to the file COPYING for more information.
%
% REFERENCE:
%  @TECHREPORT{CruzBogdPaquBierThir09,
%     Author = {Javier Cruz-Mota and Iva Bogdanova and Beno{\^\i}t Paquier and Michel Bierlaire and Jean-Philippe Thiran},
%     Institution = {Transport and Mobility Laboratory, Ecole Polytechnique F\'ed\'erale de Lausanne (EPFL)},
%     Title = {Scale Invariant Feature Transform on the Sphere: Theory and Applications},
%     Year = {2009}
%  }

    descriptor = zeros(numSpatialDivisions*numSpatialDivisions*numOrientationDivisions,1);
    descriptorOffsetX = numOrientationDivisions;
    descriptorOffsetY = numOrientationDivisions*numSpatialDivisions;
    
    [ny,nx] = size(regionAngs);
    
    numPixSpatialBinX = nx/numSpatialDivisions;
    numPixSpatialBinY = ny/numSpatialDivisions;

    for row=1:ny
        for col=1:nx
            T = eps+numOrientationDivisions*regionAngs(row,col)/(2*pi);
            binT = ceil(T);
            X = (col-1)/numPixSpatialBinX + 1/(2.0*numPixSpatialBinX);
            binX = floor(X);
            Y = (row-1)/numPixSpatialBinY + 1/(2.0*numPixSpatialBinY);
            binY = floor(Y);
            
            distT = T-binT+0.5;
            distX = X-binX-0.5;
            distY = Y-binY-0.5;
            
            adistT = abs(distT);
            adistX = abs(distX);
            adistY = abs(distY);
            
            if(distT>0)
                binT2 = binT+1;
            else
                binT2 = binT-1;
            end
            if(binT2>numOrientationDivisions)
                binT2 = binT2 - numOrientationDivisions;
            end
            if(binT2<1)
                binT2 = binT2 + numOrientationDivisions;
            end
            
            if(distX>0)
                binX2 = binX+1;
            else
                binX2 = binX-1;
            end

            if(distY>0)
                binY2 = binY+1;
            else
                binY2 = binY-1;
            end

            gaussW = exp(-2.0*((col-(nx+1.0)/2.0)^2 + (row-(ny+1.0)/2.0)^2)/(nx*ny));

            % Trilinear interpolation
            
            descriptor(    binX*descriptorOffsetX + binY*descriptorOffsetY + binT) = ...
                descriptor(binX*descriptorOffsetX + binY*descriptorOffsetY + binT) + ...
                regionMods(row,col) * gaussW * (1-adistX) * (1-adistY) * (1-adistT);

            descriptor(    binX*descriptorOffsetX + binY*descriptorOffsetY + binT2) = ...
                descriptor(binX*descriptorOffsetX + binY*descriptorOffsetY + binT2) + ...
                regionMods(row,col) * gaussW * (1-adistX) * (1-adistY) * (adistT);
            
            if(binY2>=0 && binY2<numSpatialDivisions)
                
                descriptor(    binX*descriptorOffsetX + binY2*descriptorOffsetY + binT) = ...
                    descriptor(binX*descriptorOffsetX + binY2*descriptorOffsetY + binT) + ...
                    regionMods(row,col) * gaussW * (1-adistX) * (adistY) * (1-adistT);
            
                descriptor(    binX*descriptorOffsetX + binY2*descriptorOffsetY + binT2) = ...
                    descriptor(binX*descriptorOffsetX + binY2*descriptorOffsetY + binT2) + ...
                    regionMods(row,col) * gaussW * (1-adistX) * (adistY) * (adistT);
            end

            if(binX2>=0 && binX2<numSpatialDivisions)

                descriptor(    binX2*descriptorOffsetX + binY*descriptorOffsetY + binT) = ...
                    descriptor(binX2*descriptorOffsetX + binY*descriptorOffsetY + binT) + ...
                    regionMods(row,col) * gaussW * (adistX) * (1-adistY) * (1-adistT);

                descriptor(    binX2*descriptorOffsetX + binY*descriptorOffsetY + binT2) = ...
                    descriptor(binX2*descriptorOffsetX + binY*descriptorOffsetY + binT2) + ...
                    regionMods(row,col) * gaussW * (adistX) * (1-adistY) * (adistT);
            end

            if(binX2>=0 && binX2<numSpatialDivisions && binY2>=0 && binY2<numSpatialDivisions)

                descriptor(    binX2*descriptorOffsetX + binY2*descriptorOffsetY + binT) = ...
                    descriptor(binX2*descriptorOffsetX + binY2*descriptorOffsetY + binT) + ...
                    regionMods(row,col) * gaussW * (adistX) * (adistY) * (1-adistT);

                descriptor(    binX2*descriptorOffsetX + binY2*descriptorOffsetY + binT2) = ...
                    descriptor(binX2*descriptorOffsetX + binY2*descriptorOffsetY + binT2) + ...
                    regionMods(row,col) * gaussW * (adistX) * (adistY) * (adistT);
            end
            
            
        end
    end
end
