function dD = derivSphere(Img, phiRes, thetaRes)
% dD = derivSphere(Img, phiRes, thetaRes)
%
% DESCRIPTION:
%  Computes partial derivatives with respect to phi and theta
%
% VERSION:
%  1.1
%
% INPUT:
%  Img       =  grayscale omnidirectional image mapped on the sphere
%  phiRes    =  resolution in phi
%  thetaRes  =  resolution in theta
%
% OUTPUT:
%  dD  =  derivatives with respect to phi (dD{1}) and theta (dD{2})

% AUTHOR:
%  Javier Cruz-Mota, email: javier.cruz@epfl.ch
%  Copyright (C) 2011 EPFL (Ecole Polytechnique Fédérale de Lausanne)
%  Transport and Mobility Laboratoy (TRANSP-OR)
% 
% LICENSE:
%  SIFTS is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%  Please refer to the file COPYING for more information.
%
% REFERENCE:
%  @TECHREPORT{CruzBogdPaquBierThir09,
%     Author = {Javier Cruz-Mota and Iva Bogdanova and Beno{\^\i}t Paquier and Michel Bierlaire and Jean-Philippe Thiran},
%     Institution = {Transport and Mobility Laboratory, Ecole Polytechnique F\'ed\'erale de Lausanne (EPFL)},
%     Title = {Scale Invariant Feature Transform on the Sphere: Theory and Applications},
%     Year = {2009}
%  }
    
    ImgL = circshift(Img, [0 1]);
    ImgR = circshift(Img, [0 -1]);
    
    ImgU = [Img(1,:); Img(1:end-1,:)];
    ImgD = [Img(2:end,:); Img(end,:)];

    %Partial derivative with respect to Phi
    [Iy, Ix] = size(Img);
    [~,theta] = sphgrid(Iy,Ix);
    dD{1} = (1.0./sin(theta)).*((ImgR - ImgL)/(2*phiRes));
    
    %Partial derivative with respect to Theta
    dD{2} = (ImgD - ImgU)/(2*thetaRes);

end
