function [numTotal1, numTotal2, numRepeat] = testRotationRepeatability(extremePoints1, extremePoints2, imgR, imgC, angle)
% [numTotal1,numTotal2,numRepeat] = testRotationRepeatability(extremePoints1,extremePoints2,imgR,imgC,angle)
%
% DESCRIPTION:
%  Gives statistics of repeatability between an omnidirectional image
%  mapped on the sphere and a rotation of it around the X axis. Usually,
%  the rotated image is also corrupted by additive Gaussian noise.
%  Criterion of correctness: overlap error < 0.5 (Mikolajczyk et al(2005), "A 
%  comparison of affine region detectors", International Journal of Computer
%  Vision, Vol. 65).
%
% VERSION:
%  1.1
%
% INPUT:
%  extremePoints1  =  list of extrema of the first image (original image)
%  extremePoints2  =  list of extrema of the second image (rotated image)
%  imgR            =  number of rows of the spherical grid
%  imgC            =  number of columns of the spherical grid
%  angle           =  angle of rotation between the images
%
% OUTPUT:
%  numTotal1  =  total number of points extracted from the original image
%  numTotal2  =  total number of points extracted from the rotated image
%  numRepeat  =  number of points detected in the original image and the 
%                rotated one


% AUTHOR:
%  Javier Cruz-Mota, email: javier.cruz@epfl.ch
%  Copyright (C) 2011 EPFL (Ecole Polytechnique Fédérale de Lausanne)
%  Transport and Mobility Laboratoy (TRANSP-OR)
% 
% LICENSE:
%  SIFTS is free software: you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation, either version 3 of the License, or
%  (at your option) any later version.
%  Please refer to the file COPYING for more information.
%
% REFERENCE:
%  @TECHREPORT{CruzBogdPaquBierThir09,
%     Author = {Javier Cruz-Mota and Iva Bogdanova and Beno{\^\i}t Paquier and Michel Bierlaire and Jean-Philippe Thiran},
%     Institution = {Transport and Mobility Laboratory, Ecole Polytechnique F\'ed\'erale de Lausanne (EPFL)},
%     Title = {Scale Invariant Feature Transform on the Sphere: Theory and Applications},
%     Year = {2009}
%  }
    
    numTotal1 = length(extremePoints1.octave);
    numTotal2 = length(extremePoints2.octave);

    numRepeat=0;

    [phiGrid, thetaGrid] = sphgrid(imgR, imgC);
    
    coords2 = [(2*(extremePoints2.imgOptPoints(:,1))-1)*pi/(2.0*imgR), (extremePoints2.imgOptPoints(:,2)-1)*2*pi/imgC];
    [coords2Theta, coords2Phi] = computeRotatedCoords(coords2(:,1)', coords2(:,2)', pi/2.0, pi, angle); 
    clear coords2;
    coords2 = [diag(coords2Theta),diag(coords2Phi)];
    clear coords2Theta;
    clear coords2Phi;

    dist1Computed = false;
    for i=1:numTotal1
        coords1 = [(2*extremePoints1.imgOptPoints(i,1)-1)*pi/(2*imgR), (extremePoints1.imgOptPoints(i,2)-1)*2*pi/imgC];

        for j=1:numTotal2       
            if( sphericalDistance(coords1(2), coords2(j,2), coords1(1), coords2(j,1)) < 1.5*(extremePoints1.sigma(i)+extremePoints2.sigma(j)) )
		if(~dist1Computed)
		        dist1 = sphericalDistance(coords1(2), phiGrid(:), coords1(1), thetaGrid(:));
			dist1Computed = true;
		end
                dist2 = sphericalDistance(coords2(j,2), phiGrid(:), coords2(j,1), thetaGrid(:));
                if(computeOverlapError(dist1, dist2, extremePoints1.sigma(i), extremePoints2.sigma(j)) < 0.5)
                    numRepeat = numRepeat + 1;
                    break;
                end
            end
        end
	dist1Computed = false;
    end
    
    disp('[SIFTS] Testing repeatability...');
    msg = sprintf('[SIFTS]\t %d points extracted from image 1', numTotal1);
    disp(msg);
    msg = sprintf('[SIFTS]\t %d points extracted from image 2', numTotal2);
    disp(msg);
    msg = sprintf('[SIFTS]\t %d corresponding points extracted from image 1 and 2', numRepeat);
    disp(msg);
    msg = sprintf('[SIFTS]\t %.2f%% of repeatability', 100.0*numRepeat/min(numTotal1,numTotal2));
    disp(msg);
    msg = sprintf('[SIFTS] Done!\n');
    disp(msg);
    
end
